      subroutine boundsep(n,irc,energy,wfn,l,rveff,h,v0,v0p,proj,nmx,   &
        hij,qij,mxb,many,a,b,y,u,d1,d2,p1,emin,emax,niter,ierr,         &
        odphi,alwfn,nz)
!  pgm to solve separable radial schroedinger equation for bound state
!    near the energy 'energy' and at angular momentum l
!
! determines both smooth wavefunction wfn and all-electron wavefunction
!   alwfn
!
!    with potential rveff/r, given in uniform mesh of n points
!   r=i*h, i=1,...n-1 ;assuming p(r)=C*r**(l+1)*polynomial(r) for r==0;
!                               p((n+1)*h)=0
!
!  uses Noumerov algorithm
!
!  For l=0,1 corrections are needed to approximate wfn(r=0)
!     These depend upon:
!         e0 (current guess of energy eigenvalue)
!         l,nz
!         v(0) == v0 electronic potential at r=0
!         v'(0) == v0p derivative of electronic potential at r=0
!
!  proj == projector functions
!  odphi == phi - tphi
!  hij and qij == hamiltonianian and overlap matrix elements
!
      implicit real*8 (a-h,o-z)
      parameter (convre=1.d-10,vlrg=1.d30,inc=10)
      dimension rveff(n),wfn(n),proj(nmx,many),y(nmx,many),u(nmx)
      dimension a(mxb,mxb),b(mxb),hij(mxb,mxb),qij(mxb,mxb)
      dimension d1(nmx),d2(nmx),p1(nmx),alwfn(n),odphi(nmx,many)
      err=n*(nz*h)**5
      convrez=convre
      if (nz.gt.0) convrez=convre*nz
      ierr=0
      ifac=9
      iter=0
      if (n.gt.nmx) then
       write(6,*) 'error in boundsep -- n,nmx = ',n,nmx
       stop
       endif
!
       if (emax.gt.0.d0) emax=0.d0
       best=1.d10
       if (energy.lt.emin) energy=emin+err
       if (energy.gt.emax) energy=emax
       energy0=energy
 3001 continue
!xx   write(6,*) 'iter,energy',iter,energy
!     write(6,*) 'emin,max',emin,emax
!
!  perform inward integration until match point
      match=min0(irc+100,n-10)
!xx   write(6,*) ' match point = ',match
      mmatch=match+1
!
      call inwardh(n,energy,p1,l,rveff,h,match,rin,nmx,d1,d2)
!
!  perform outward integration until match point -- it is assumed
!   that projector functions proj are zero for r>r(match)
!
      noz=0
      call outwardh(mmatch,energy,u,l,noz,rveff,h,v0,v0p,node,          &
          nmx,d1,d2)
!
      do ib=1,many
      call outwardi(mmatch,energy,y(1,ib),l,rveff,proj(1,ib),h,         &
        nmx,d1,d2)
      enddo
!
      do ib=1,many
      sum=0.d0
       do ic=1,many
       sum=sum+(hij(ib,ic)-energy*qij(ib,ic))*                          &
                 overlap(mmatch,h,proj(1,ic),u)
       enddo
      b(ib)=-sum
!     write(6,*) 'ib,sum',ib,sum
      enddo
!
      do ia=1,many
      do ib=1,many
      sum=0.d0
      if (ia.eq.ib) sum=1.d0
       do ic=1,many
       sum=sum+(hij(ia,ic)-energy*qij(ia,ic))*                          &
                 overlap(mmatch,h,proj(1,ic),y(1,ib))
       enddo
      a(ia,ib)=sum
!     write(6,*) 'ia,ib,sum',ia,ib,sum
      enddo
      enddo
!
      call linsol(a,b,mxb,many,1)
!     write(6,*) 'b(ib)=',(b(ib),ib=1,many)
      wfn(1:mmatch)=u(1:mmatch) !call dcopy(mmatch,u,1,wfn,1)
      do ib=1,many
      wfn(1:mmatch)=wfn(1:mmatch)+b(ib)*y(1:mmatch,ib)
              !   call daxpy(mmatch,b(ib),y(1,ib),1,wfn,1)
      enddo
!
      rout=(wfn(match+1)-wfn(match-1))/(2*h*wfn(match))
!xx   write(6,*) 'node,match,rin,rout',node,match,rin,rout
!   -- estimate correction
          do j=1,match
          wfn(j)=wfn(j)/wfn(match)
          enddo
          do j=match,n
          wfn(j)=p1(j)/p1(match)
          enddo
!  normalization
          scale=1.d0/sepnorm(n,h,wfn,nmx,many,proj,b,mxb,qij)
          dele=(rout-rin)*scale
!xx       write(6,*) 'dele,scale',dele,scale
          x=abs(dele)
          if (x.lt.best) then
          energy0=energy
          best=x
          endif
        if (abs(dele).le.convrez) go to 5001
          energy=energy+dele
! if energy is out of range, pick random energy in correct range
          if (emin-energy.gt.convrez.or.energy-emax.gt.convrez)         &
              energy=emin+(emax-emin)*ranx()+err
           ifac=2
          iter=iter+1
          if (iter.gt.niter) then
          ierr=ifac
          write(6,*) 'no convergence in boundsep',l,dele,energy
          write(6,*) ' best guess of eig, dele = ',energy0,best
!         write(6,*) 'returning from boundsep -- ierr=',ierr
          return
          endif
          go to 3001
 5001 continue
!  eigenvalue found
      scale=sqrt(scale)
      wfn(1:n)=wfn(1:n)*scale   ! call dscal(n,scale,wfn,1)
!
      call smoothtoae(n,h,wfn,alwfn,nmx,many,proj,odphi)
!
      ierr=0
!         write(6,*) 'returning from boundsep -- ierr=',ierr
      return
       end
