      subroutine potential(n,h,nz,q,den,rv,ecoul,etxc,eexc,             &
       v0,v0p,a,b,c,d,itype)
!  calculate veff for density functional theory from electron density
!  nz is nuclear charge
!  den(n) is electron density * (4*pi*r**2)
!  rv(n) is returned as veff * r
!  q is total electron charge
!  ecoul and etxc are coulomb and exchange-correlation contributions
!    to the total energy
!  eexc is the total exchange energy (int(den*exc))
!  v0=v(0)
!  v0p=dv/dr(0)
!  a(n), b(n), c(n), d(n) are work arrays
!  if itype=15 (gga case) need to calculate additional functions:
!       gd=abs(grad(density))/density
!       ggd=laplacian(density)/density
!       gggd=grad(density)*grad(abs(grad(density)))/density**2
!       for r<r0 (fixed in parameter statement), replace density by Gaussian
!         form so that Laplacian does not diverge
!
  Use GlobalConstants

       implicit real*8 (a-h,o-z)
       parameter (r0=0.01d0)
       dimension den(n),rv(n),a(n),b(n),c(n),d(n)
       i0=r0/h
       i0=max0(i0,2)
       r00=i0*h
       fpi = 4 * pi      !fpi=16*datan(1.d0)

!
!  Coulomb contribution
!
       call poisson(n,h,q,den,rv,a,b,ecoul)
!      do i=1,n
!      write(6,*) i, 'poisson',rv(i)
!      enddo
!
!
!  Exchange-correlation contribution  exc-vxc
!
        call exchcor(n,h,den,d,etxc,eexc,a,b,c,itype)
!
        th=1
        rv(1:n)=rv(1:n)+d(1:n)  !call daxpy(n,th,d,1,rv,1)
!
!  add in nuclear contribution and calculate v0 and v0p
!
        do i=1,15
        r=i*h
        a(i)=rv(i)/r
        enddo
        call nderiv(h,a,b,15)
        v0=a(3)+3*(a(1)-a(2))
        v0p=b(3)+3*(b(1)-b(2))
!!!!!!!!!!!!!! for GGA form v0p diverges -- so set v0p=0
!!!!!!!!!!!!!!        if (itype.ge.15) v0p=0.d0
        write(6,*) 'v0, v0p = ', v0,v0p
!       rewind 8
        do i=1,n
        rv(i)=rv(i)-2*nz
!       r=i*h
!       write(8,800) i,r,den(i),rv(i)
! 800   format(i5,1p3e15.7)
        enddo
        return
         end
